local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Group = import('/lua/maui/group.lua').Group
local GameMain = import('/lua/ui/game/gamemain.lua')
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local Dragger = import('/lua/maui/dragger.lua').Dragger
local borders = import('/lua/ui/game/borders.lua').GetMapGroup()
local ToolTip = import('/lua/ui/game/tooltip.lua')
local EffectHelpers = import('/lua/maui/effecthelpers.lua')
local Prefs = import('/lua/user/prefs.lua')--preferences

--AutoSelection = true
local panel = false
local mouseover = false
local pToggle = false
local savedPrefs

function CreatePanel(parent)
local savedPrefs = Prefs.GetFromCurrentProfile("UIAdvGrpManager_settings")
	local panel = Bitmap(parent)
	panel:SetTexture('/mods/AlsComboPack/textures/panel.dds')
	panel.Depth:Set(100)
	if savedPrefs.Left < GetFrame(0).Right and savedPrefs.Top < GetFrame(0).Bottom then
		LayoutHelpers.AtLeftTopIn(panel, GetFrame(0), savedPrefs.Left, savedPrefs.Top)
	else
		LayoutHelpers.AtLeftTopIn(panel, GetFrame(0), savedPrefs.Left, savedPrefs.Top)
	end
panel.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			local drag = Dragger()
			local offY = event.MouseY - self.Top()
			drag.OnMove = function(dragself, x, y)
				self.Top:Set(y - offY)
				self.Left:Set(function() return parent.Left() + 18 end)
				if self.Top() ~= panelToggle.Top() then
				panelToggle:Destroy()
				CreatePanelToggle(self.Top())
				end
				GetCursor():SetTexture(UIUtil.GetCursor('MOVE_WINDOW'))
			end
			drag.OnRelease = function(dragself)
				local tempPrefs = Prefs.GetFromCurrentProfile("UIAdvGrpManager_settings")
				tempPrefs.Left = self.Left()
				tempPrefs.Top = self.Top()
				Prefs.SetToCurrentProfile("UIAdvGrpManager_settings", tempPrefs)
				Prefs.SavePreferences()
				GetCursor():Reset()
				drag:Destroy()
			end
			PostDragger(self:GetRootFrame(), event.KeyCode, drag)
		elseif event.Type == 'MouseExit' then
			CloseToolTip()
		end
	end
	return panel
end

function CreatePanelToggle(yaxisoff)
local savedPrefs = Prefs.GetFromCurrentProfile("UIAdvGrpManager_settings")
	panelToggle = UIUtil.CreateCheckbox(borders, 
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_dis.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_dis.dds'))
    LayoutHelpers.AtLeftTopIn(panelToggle, borders, 2, (yaxisoff))
  panelToggle.Height:Set(16)
	panelToggle.Depth:Set(100)
    panelToggle.OnCheck = function(self, checked)
        if checked then
			hideEffect(panel, squadPanel, splitPanel)
		else
			showEffect(panel, squadPanel, splitPanel)
		end
    end
	panelToggle.oldHandleEvent = panelToggle.HandleEvent
	panelToggle.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			CloseToolTip()
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize Advanced Grouping panel."}, true)
			else
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize Advanced Grouping panel."}, true)
			end
		elseif event.Type == 'MouseEnter' then
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize Advanced Grouping panel."}, true)
			else
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize Advanced Grouping panel."}, true)
			end
		elseif event.Type == 'MouseExit' then
			CloseToolTip()
		end
		panelToggle.oldHandleEvent(self, event)
	end
end

function CreateSquadControls(parent)
local savedPrefs = Prefs.GetFromCurrentProfile("UIAdvGrpManager_settings")
	local squadgroup = Group(parent)
	squadgroup.Left:Set(parent.Left)
	squadgroup.Top:Set(parent.Top)
	squadgroup.Right:Set(parent.Right)
	squadgroup.Height:Set(20)

	local autoselectbtn = Checkbox(squadgroup,
		'/mods/AlsComboPack/textures/autoselectbtn_up.dds',
		'/mods/AlsComboPack/textures/autoselectbtn_up.dds',
		'/mods/AlsComboPack/textures/autoselectbtn_over.dds',
		'/mods/AlsComboPack/textures/autoselectbtn_over.dds',
		'/mods/AlsComboPack/textures/autoselectbtn_up.dds',
		'/mods/AlsComboPack/textures/autoselectbtn_up.dds')
	LayoutHelpers.AtLeftTopIn(autoselectbtn, squadgroup, 3, 2)
	autoselectbtn.title = UIUtil.CreateText(autoselectbtn, "Squads", 9)
	LayoutHelpers.AtCenterIn(autoselectbtn.title, autoselectbtn, 1, 0)

	autoselectbtn.OnClick = function(self) end
	autoselectbtn.oldHandleEvent = autoselectbtn.HandleEvent
	autoselectbtn.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
      if self:IsChecked() then
        AutoSelection = false
        savedPrefs.AutoSelection = false
        autoselectbtn.title:SetColor('e61b1b')
				self:SetCheck(false)
			else
        AutoSelection = true
        savedPrefs.AutoSelection = true
        autoselectbtn.title:SetColor('2ec74d')
        self:SetCheck(true)
      end
    Prefs.SetToCurrentProfile("UIAdvGrpManager_settings", savedPrefs)
    Prefs.SavePreferences()
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Auto Squad Selection", Body = "Toggle automatic selection of squads when any part of a squad is selected."})
		end
		autoselectbtn.oldHandleEvent(self, event)
	end

	if savedPrefs.AutoSelection == false then
    AutoSelection = false
    autoselectbtn:SetCheck(false)
    autoselectbtn.title:SetColor('e61b1b')
  else
    AutoSelection = true
    autoselectbtn:SetCheck(true)
    autoselectbtn.title:SetColor('2ec74d')
	end
	
	local squadbtn = Button(squadgroup,
		'/mods/AlsComboPack/textures/splitbtn_up.dds',
		'/mods/AlsComboPack/textures/splitbtn_down.dds',
		'/mods/AlsComboPack/textures/splitbtn_over.dds',
		'/mods/AlsComboPack/textures/splitbtn_up.dds')
	LayoutHelpers.AtLeftTopIn(squadbtn, squadgroup, 43, 2)
	squadbtn.title = UIUtil.CreateText(squadbtn, "C", 10)
	squadbtn.title:SetColor('338833')
	LayoutHelpers.AtCenterIn(squadbtn.title, squadbtn, 2, 1)
	squadbtn.oldHandleEvent = squadbtn.HandleEvent
	squadbtn.HandleEvent = function(self, event)
    if event.Type == 'ButtonPress' then
      import("/mods/AlsComboPack/Modules/selectionsplit.lua").SquadCreator()
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Create Squad", Body = "Create a squad from the current selection."})
		end
		squadbtn.oldHandleEvent(self, event)
	end

	local disbandbtn = Button(squadgroup,
		'/mods/AlsComboPack/textures/splitbtn_up.dds',
		'/mods/AlsComboPack/textures/splitbtn_down.dds',
		'/mods/AlsComboPack/textures/splitbtn_over.dds',
		'/mods/AlsComboPack/textures/splitbtn_up.dds')
	LayoutHelpers.AtLeftTopIn(disbandbtn, squadgroup, 60, 2)
	disbandbtn.title = UIUtil.CreateText(disbandbtn, "X", 10)
	disbandbtn.title:SetColor('883333')
	LayoutHelpers.AtCenterIn(disbandbtn.title, disbandbtn, 2, 1)
	disbandbtn.oldHandleEvent = disbandbtn.HandleEvent
	disbandbtn.HandleEvent = function(self, event)
    if event.Type == 'ButtonPress' then
		import("/mods/AlsComboPack/Modules/selectionsplit.lua").Disband()    
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Disband Squads", Body = "Disband any currently selected squads."})
		end
		disbandbtn.oldHandleEvent(self, event)
	end
	return squadgroup
end


function CreateSplitControls(parent)
	local splitgroup = Group(parent)
	splitgroup.Left:Set(parent.Left)
	splitgroup.Top:Set(function() return parent.Top() + 20 end)
	splitgroup.Right:Set(parent.Right)
	splitgroup.Height:Set(30)
	local title = UIUtil.CreateText(splitgroup, " Selection split:", 9)
	title.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Split Selection", Body = "Click a numbered button to split the current selection into X subgroups. Press and hold to drag panel."})
		end
	end
	local btngroup = Group(splitgroup)
	btngroup.Height:Set(20)
	btngroup.Width:Set(80)
	local btn = {}
	for i = 2, 5 do
		local index = i
		btn[index] = Button(btngroup,
			'/mods/AlsComboPack/textures/splitbtn_up.dds',
			'/mods/AlsComboPack/textures/splitbtn_down.dds',
			'/mods/AlsComboPack/textures/splitbtn_over.dds',
			'/mods/AlsComboPack/textures/splitbtn_up.dds')
		if index == 2 then
			LayoutHelpers.AtLeftTopIn(btn[index], btngroup)
		else
			LayoutHelpers.RightOf(btn[index], btn[index-1], 1)
		end
		btn[index].title = UIUtil.CreateText(btn[index], i, 10)
		LayoutHelpers.AtCenterIn(btn[index].title, btn[index], 2, 1)
		btn[index].oldHandleEvent = btn[index].HandleEvent
		btn[index].HandleEvent = function(self, event)
      if event.Type == 'ButtonPress' then
        import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(index)     
			elseif event.Type == 'MouseEnter' then
				CreateToolTip(self, {Title = "Split Selection", Body = "Split current selection into "..index.." subgroups."})
			end
			btn[index].oldHandleEvent(self, event)
		end
	end
	LayoutHelpers.AtLeftTopIn(title, splitgroup, 5, 1)
	LayoutHelpers.AtLeftTopIn(btngroup, splitgroup, 3, 11)	
	return splitgroup
end

function CreateToolTip(control, data, togglebutton)
	if mouseover then mouseover:Destroy() mouseover = false end
	local text = data.Title
	local body = data.Body
	mouseover = ToolTip.CreateExtendedToolTip(control, text, body)
	mouseover.Top:Set(function() return panel.Top() + 2 end)
  mouseover.Left:Set(function() return panel.Right() + 1 end)
	mouseover:SetAlpha(0, true)
	mouseover:SetNeedsFrameUpdate(true)
	
	local delayTime = 0.5
	local totaltime = 0
	mouseover.OnFrame = function(self, deltaTime)
		if totaltime > delayTime then
			local alpha = self:GetAlpha() + (deltaTime * 4)
			if alpha < 1 then
				self:SetAlpha(alpha, true)
			else
				self:SetAlpha(1, true)
				self:SetNeedsFrameUpdate(false)
			end
		end
		totaltime = totaltime + deltaTime
	end
end
		
function CloseToolTip()
	if mouseover then mouseover:Destroy() mouseover = false end
end

function hideEffect(control, subcontrol1, subcontrol2)
	control:DisableHitTest(true)
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.1
	local time2 = 0.2
	local minHeight = 16
    local timeAccum = 0

	EffectHelpers.FadeOut(subcontrol1, timefade)
	EffectHelpers.FadeOut(subcontrol2, timefade)
	
    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
            self:SetNeedsFrameUpdate(false)
            self:Hide()
			self.Right:Set(function() return self.Left() + 3 end)
		elseif timeAccum >= time1 then
			self.Right:Set(function() return self.Left() + self.BitmapWidth()*((time2+time1 - timeAccum)/(time2)) end)
			self.Bottom:Set(function() return self.Top() + minHeight end)
		else
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((time1 - timeAccum)/(time1)) end )
		end
    end
end

function showEffect(control, subcontrol1, subcontrol2)
	control:Show()
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.2
	local time2 = 0.1
	local minHeight = 16
    local timeAccum = 0

    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
			self:EnableHitTest(true)
            self:SetNeedsFrameUpdate(false)
			LayoutHelpers.ResetRight(self)
			LayoutHelpers.ResetBottom(self)
			EffectHelpers.FadeIn(subcontrol1, timefade)
			EffectHelpers.FadeIn(subcontrol2, timefade)
		elseif timeAccum >= time1 then
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((timeAccum - time1)/(time2)) end )
			self.Right:Set(function() return self.Left() + self.BitmapWidth() end)
        else
			self.Right:Set(function() return self.Left() + self.BitmapWidth()*((timeAccum)/(time1)) end)
        end
    end
end

function Init()
	local savedPrefs = Prefs.GetFromCurrentProfile("UIAdvGrpManager_settings")
	if not savedPrefs then
		savedPrefs = {
			Left = 18,
			Top = 450,
			AutoSelection = true,
		}
		Prefs.SetToCurrentProfile("UIAdvGrpManager_settings", savedPrefs)
		Prefs.SavePreferences()
	end
	panel = CreatePanel(borders)
	squadPanel = CreateSquadControls(panel)
	splitPanel = CreateSplitControls(panel)
	pToggle = CreatePanelToggle(savedPrefs.Top)
	
	local newSelectionsMap = {
    ['Alt-1']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt1")'},
    ['Alt-2']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt2")'},
    ['Alt-3']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt3")'},
    ['Alt-4']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt4")'},
    ['Alt-5']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt5")'},
    ['Alt-6']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt6")'},
    ['Alt-7']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt7")'},
    ['Alt-8']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt8")'},
    ['Alt-9']              = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SplitSelector("alt9")'},
	['G']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").GroupChecker()'},
	['Ctrl-G']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SquadCreator()'},
	['Ctrl-D']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").Disband()'},
#	['Alt-X']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SquadCheck()'},
	['Ctrl-Alt-1']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(1)'},
	['Ctrl-Alt-2']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(2)'},
	['Ctrl-Alt-3']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(3)'},
	['Ctrl-Alt-4']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(4)'},
	['Ctrl-Alt-5']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(5)'},
	['Ctrl-Alt-6']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(6)'},
	['Ctrl-Alt-7']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(7)'},
	['Ctrl-Alt-8']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(8)'},
	['Ctrl-Alt-9']         = {action =  'UI_Lua import("/mods/AlsComboPack/Modules/selectionsplit.lua").SelectionSplitter(9)'},
	}
	IN_AddKeyMapTable(newSelectionsMap)
end
