local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Group = import('/lua/maui/group.lua').Group
local GameMain = import('/lua/ui/game/gamemain.lua')
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local Dragger = import('/lua/maui/dragger.lua').Dragger
local StatusBar = import('/lua/maui/statusbar.lua').StatusBar
local Prefs = import('/lua/user/prefs.lua')--preferences
local IntegerSlider = import('/lua/maui/slider.lua').IntegerSlider
local borders = import('/lua/ui/game/borders.lua').GetMapGroup()
local ToolTip = import('/lua/ui/game/tooltip.lua')
local EffectHelpers = import('/lua/maui/effecthelpers.lua')

local panel = false
local mouseover = false
local pToggle = false
local savedPrefs = Prefs.GetFromCurrentProfile("UIMassFabManager_settings")

local faxThread
local switchThread

local fabList = {}
local fab2List = {}

local tmpicon = nil

local AutoUpdateFabs = false
local percentOnSlider = false
local totalFabs = false
local totalOFabs = false

local OnTime = 15
local OffTime = 10
local OnStorage = 0.5
local OffStorage = 0.25


local HelpText = {
	OnTime = {
		Title = "On Time",
		Body = "If energy stored will last longer than OnTime seconds, and energy storage is above OnStorage% full then fabricators will start turning on.",
	},
	OffTime = {
		Title = "Off Time",
		Body = "If energy stored will last less than OffTime seconds, then fabricators will start turning off.",
	},
	OnStorage = {
		Title = "On Storage",
		Body = "If energy stored will last longer than OnTime seconds, and energy storage is over OnStorage% full then fabricators will start turning on.",
	},
	OffStorage = {
		Title = "Off Storage",
		Body = "If energy stored is below OffStorage% then fabricators will start turning off.",
	},
	AddBtn = {
		Title = "Add Fabricators",
		Body = "Sets currently selected fabricators to auto.",
	},
	RemoveBtn = {
		Title = "Remove Fabricators",
		Body = "Disables auto on currently selected fabricators.",
	},
	AddAllBtn = {
		Title = "Add All Fabricators",
		Body = "Left Click: Sets all fabricators to auto. Right Click: Automatically sets any new fabricators to auto.",
	},
	OptionsBtn = {
		Title = "Options",
		Body = "Configure fabricators auto behavior.",
	},
	Slider = {
		Title = "Fabricators On",
		Body = "Shows what percentage of fabricators are currently on.",
	},
	TotalFabs = {
		Title = "Total Fabricators",
		Body = "Shows the total number of fabricators being monitored.",
	},
	mficons = {
		Title = "Mass Fabricator Manager",
		Body = "Shows the total number of fabricators. Press and hold to drag panel.",
	},
}

function GetFactionIcon()
  while tmpicon == nil do
	local bilist = GetSelectedUnits() or {}
    if bilist ~= nil then
      for i, unit in bilist do
        if unit:IsInCategory('AEON') then
        LOG("AEON")
          tmpicon = ('/mods/AlsComboPack/textures/UAB1303_build_btn.dds')
        elseif unit:IsInCategory('UEF') then
        LOG("UEF")
          tmpicon = ('/mods/AlsComboPack/textures/UEB1303_build_btn.dds')
        elseif unit:IsInCategory('CYBRAN') then
        LOG("CYBRAN")
          tmpicon = ('/mods/AlsComboPack/textures/URB1303_build_btn.dds')
        elseif unit:IsInCategory('SERAPHIM') then
        LOG("SERAPHIM")
          tmpicon = ('/mods/AlsComboPack/textures/XSB1303_build_btn.dds')
        end
      end
    else
      WaitSeconds(0.3)
    end
    WaitSeconds(0.3)
  end
  panel = CreatePanel(borders)
	Massfabpanel = CreateFabControls(panel)
	pToggle = CreatePanelToggle(savedPrefs.Top)
	WaitSeconds(0.3)
	switchThread = ForkThread(SwitchFabs)
end

function CreatePanel(parent)
	local panel = Bitmap(parent)
	panel:SetTexture('/mods/AlsComboPack/textures/panel.dds')
	panel.Depth:Set(100)
	panel.Height:Set(51)
  panel.Width:Set(90)
	if savedPrefs.Left < GetFrame(0).Right and savedPrefs.Top < GetFrame(0).Bottom then
		LayoutHelpers.AtLeftTopIn(panel, GetFrame(0), savedPrefs.Left, savedPrefs.Top)
	else
		LayoutHelpers.AtLeftTopIn(panel, GetFrame(0), savedPrefs.Left, savedPrefs.Top)
	end
	panel.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			local drag = Dragger()
			local offY = event.MouseY - self.Top()
			drag.OnMove = function(dragself, x, y)
				self.Top:Set(y - offY)
				self.Left:Set(function() return parent.Left() + 18 end)
				if self.Top() ~= panelToggle.Top() then
				panelToggle:Destroy()
				CreatePanelToggle(self.Top())
				end
				GetCursor():SetTexture(UIUtil.GetCursor('MOVE_WINDOW'))
			end
			drag.OnRelease = function(dragself)
				local tempPrefs = Prefs.GetFromCurrentProfile("UIMassFabManager_settings")
				tempPrefs.Left = self.Left()
				tempPrefs.Top = self.Top()
				Prefs.SetToCurrentProfile("UIMassFabManager_settings", tempPrefs)
				Prefs.SavePreferences()
				GetCursor():Reset()
				drag:Destroy()
			end
			PostDragger(self:GetRootFrame(), event.KeyCode, drag)
		elseif event.Type == 'MouseExit' then
			CloseToolTip()
		end
	end
	return panel
end

function CreatePanelToggle(yaxisoff)
	panelToggle = UIUtil.CreateCheckbox(borders, 
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_dis.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_dis.dds'))
    LayoutHelpers.AtLeftTopIn(panelToggle, borders, 2, (yaxisoff))
  panelToggle.Height:Set(16)
	panelToggle.Depth:Set(100)
    panelToggle.OnCheck = function(self, checked)
        if checked then
			hideEffect(panel, Massfabpanel)
		else
			showEffect(panel, Massfabpanel)
		end
    end
	panelToggle.oldHandleEvent = panelToggle.HandleEvent
	panelToggle.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			CloseToolTip()
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize the Mass Fabricator panel."}, true)
			else
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize the Mass Fabricator panel."}, true)
			end
		elseif event.Type == 'MouseEnter' then
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize the Mass Fabricator panel."}, true)
			else
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize the Mass Fabricator panel."}, true)
			end
		elseif event.Type == 'MouseExit' then
			CloseToolTip()
		end
		panelToggle.oldHandleEvent(self, event)
	end
end

function CreateFabControls(parent)
  local fabgroup = Group(parent)
	fabgroup.Left:Set(parent.Left)
	fabgroup.Top:Set(parent.Top)
	fabgroup.Right:Set(parent.Right)
	fabgroup.Bottom:Set(parent.Bottom)
	
	  local AddFabButton = Button(fabgroup, '/mods/AlsComboPack/textures/add_up.dds', '/mods/AlsComboPack/textures/add_down.dds', '/mods/AlsComboPack/textures/add_over.dds', '/mods/AlsComboPack/textures/add_up.dds', "UI_Menu_MouseDown_Sml", "UI_Menu_MouseDown_Sml")
    LayoutHelpers.AtLeftTopIn(AddFabButton, fabgroup, 4, 13)
    local RemoveFabButton = Button(fabgroup, '/mods/AlsComboPack/textures/remove_up.dds', '/mods/AlsComboPack/textures/remove_down.dds', '/mods/AlsComboPack/textures/remove_over.dds', '/mods/AlsComboPack/textures/remove_up.dds', "UI_Menu_MouseDown_Sml", "UI_Menu_MouseDown_Sml")
    LayoutHelpers.AtLeftTopIn(RemoveFabButton, fabgroup, 22, 13)
    local AddAllFabsButton = Checkbox(fabgroup, '/mods/AlsComboPack/textures/addall_up.dds', '/mods/AlsComboPack/textures/addall_down.dds', '/mods/AlsComboPack/textures/addall_over.dds', '/mods/AlsComboPack/textures/addall_over.dds', '/mods/AlsComboPack/textures/addall_up.dds', '/mods/AlsComboPack/textures/addall_up.dds', "UI_Menu_MouseDown_Sml", "UI_Menu_MouseDown_Sml")
    LayoutHelpers.AtLeftTopIn(AddAllFabsButton, fabgroup, 4, 30)
    local OptionsButton = Button(fabgroup, '/mods/AlsComboPack/textures/options_up.dds', '/mods/AlsComboPack/textures/options_down.dds', '/mods/AlsComboPack/textures/options_over.dds', '/mods/AlsComboPack/textures/options_up.dds', "UI_Menu_MouseDown_Sml", "UI_Menu_MouseDown_Sml")
    LayoutHelpers.AtLeftTopIn(OptionsButton, fabgroup, 22, 30)

    local mficon = Bitmap(fabgroup, tmpicon)
    LayoutHelpers.AtLeftTopIn(mficon, fabgroup, 41, 4)
    mficon.Height:Set(44)
    mficon.Width:Set(44)

	mficon.oldHandleEvent = mficon.HandleEvent
	mficon.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.mficons.Title, Body = HelpText.mficons.Body})
		end
		mficon.oldHandleEvent(self, event)
	end
	
	totalOFabs = UIUtil.CreateText(fabgroup, "0", 18, UIUtil.bodyFont)
	LayoutHelpers.AtLeftTopIn(totalOFabs, fabgroup, 43, 4)
	totalOFabs:SetColor('ffffffff')
	totalOFabs:SetDropShadow(true)
  totalOFabs.Depth:Set(function() return mficon.Depth() + 1 end)	

	totalOFabs.oldHandleEvent = totalOFabs.HandleEvent
	totalOFabs.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Total Fabricators", Body = "Shows current total number of fabricators."})
		end
		totalOFabs.oldHandleEvent(self, event)
	end

	AddFabButton.oldHandleEvent = AddFabButton.HandleEvent
	AddFabButton.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			local selection = GetSelectedUnits() or {}
			for i, unit in selection do
				if unit:IsInCategory('MASSFABRICATION') and unit:IsInCategory('STRUCTURE') then
					fabList[unit:GetEntityId()] = unit
				end
			end
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.AddBtn.Title, Body = HelpText.AddBtn.Body})
		end
		AddFabButton.oldHandleEvent(self, event)
	end

	RemoveFabButton.oldHandleEvent = RemoveFabButton.HandleEvent
	RemoveFabButton.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			local selection = GetSelectedUnits() or {}
			for i, unit in selection do
				if unit:IsInCategory('MASSFABRICATION') and unit:IsInCategory('STRUCTURE') then
					fabList[unit:GetEntityId()] = nil
				end
			end
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.RemoveBtn.Title, Body = HelpText.RemoveBtn.Body})
		end
		RemoveFabButton.oldHandleEvent(self, event)
	end
	
	AddAllFabsButton.OnClick = function(self) end
	AddAllFabsButton.oldHandleEvent = AddAllFabsButton.HandleEvent
	AddAllFabsButton.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			if event.Modifiers.Left then
				local selection = GetSelectedUnits()
				local mflist = UISelectionByCategory("MASSFABRICATION * STRUCTURE", false, false, false, false)
				local mflist = GetSelectedUnits() or {}
				for i, unit in mflist do
					fabList[unit:GetEntityId()] = unit
				end
				SelectUnits(selection)
			else
				if self:IsChecked() then
					AutoUpdateFabs = false
					self:SetCheck(false)
					savedPrefs.AddAllFabsButton = false
				else
					AutoUpdateFabs = true
					self:SetCheck(true)
					savedPrefs.AddAllFabsButton = true
				end
      Prefs.SetToCurrentProfile("UIMassFabManager_settings", savedPrefs)
      Prefs.SavePreferences()
			end
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.AddAllBtn.Title, Body = HelpText.AddAllBtn.Body})
		end
		AddAllFabsButton.oldHandleEvent(self, event)
	end

	if savedPrefs.AddAllFabsButton == true then
    AddAllFabsButton:SetCheck(true)
    AutoUpdateFabs = true
	end
	
	OptionsButton.oldHandleEvent = OptionsButton.HandleEvent
	OptionsButton.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			OptionsPanel(panel)
		elseif event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.OptionsBtn.Title, Body = HelpText.OptionsBtn.Body})
		end
		OptionsButton.oldHandleEvent(self, event)
	end

	percentOnSlider = StatusBar(fabgroup, 0, 1, false, false, '/mods/AlsComboPack/textures/slider_bg.dds', '/mods/AlsComboPack/textures/slider_fg.dds', false)
    LayoutHelpers.AtLeftTopIn(percentOnSlider, fabgroup, 4, 3)
	percentOnSlider:SetMinimumSlidePercentage(5/29)
	percentOnSlider.oldHandleEvent = percentOnSlider.HandleEvent
	percentOnSlider.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.Slider.Title, Body = HelpText.Slider.Body})
		end
		percentOnSlider.oldHandleEvent(self, event)
	end
	
	totalFabs = UIUtil.CreateText(fabgroup, "0", 10, UIUtil.bodyFont)
	LayoutHelpers.AtLeftTopIn(totalFabs, fabgroup, 30, 1)
	totalFabs.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(fabgroup, {Title = HelpText.TotalFabs.Title, Body = HelpText.TotalFabs.Body})
		end
		
	end
	return fabgroup
end

function CreateToolTip(control, data, togglebutton)
	if mouseover then mouseover:Destroy() mouseover = false end
	local text = data.Title
	local body = data.Body
	mouseover = ToolTip.CreateExtendedToolTip(control, text, body)
	mouseover.Top:Set(function() return panel.Top() + 2 end)
	mouseover.Left:Set(function() return panel.Right() + 1 end)
	mouseover:SetAlpha(0, true)
	mouseover:SetNeedsFrameUpdate(true)
	
	local delayTime = 0.5
	local totaltime = 0
	mouseover.OnFrame = function(self, deltaTime)
		if totaltime > delayTime then
			local alpha = self:GetAlpha() + (deltaTime * 4)
			if alpha < 1 then
				self:SetAlpha(alpha, true)
			else
				self:SetAlpha(1, true)
				self:SetNeedsFrameUpdate(false)
			end
		end
		totaltime = totaltime + deltaTime
	end
end
		
function CloseToolTip()
	if mouseover then mouseover:Destroy() mouseover = false end
end

function hideEffect(control, subcontrol1)
	control:DisableHitTest(true)
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.1
	local time2 = 0.2
	local minHeight = 16
    local timeAccum = 0
    
	EffectHelpers.FadeOut(subcontrol1, timefade)
	
    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
            self:SetNeedsFrameUpdate(false)
            self:Hide()
			self.Right:Set(function() return self.Left() + 3 end)
		elseif timeAccum >= time1 then
			self.Right:Set(function() return self.Left() + self.BitmapWidth()*((time2+time1 - timeAccum)/(time2)) end)
			self.Bottom:Set(function() return self.Top() + minHeight end)
		else
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((time1 - timeAccum)/(time1)) end )
		end
    end
end

function showEffect(control, subcontrol1)
	control:Show()
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.2
	local time2 = 0.1
	local minHeight = 16
    local timeAccum = 0

    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
			self:EnableHitTest(true)
            self:SetNeedsFrameUpdate(false)
			LayoutHelpers.ResetRight(self)
			LayoutHelpers.ResetBottom(self)
			EffectHelpers.FadeIn(subcontrol1, timefade)
		elseif timeAccum >= time1 then
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((timeAccum - time1)/(time2)) end )
			self.Right:Set(function() return self.Left() + self.BitmapWidth() end)
        else
			self.Right:Set(function() return self.Left() + self.BitmapWidth()*((timeAccum)/(time1)) end)
        end
    end
end

function cleanLists(tabList)
	for i, unit in (tabList) do
		if unit:IsDead() then
			(tabList)[i] = nil
		end
	end
end

function SwitchFabs()
	while true do
		
		local selection = GetSelectedUnits()
		if selection then
      WaitSeconds(0.3)
    else
      local mf2list = UISelectionByCategory("MASSFABRICATION * STRUCTURE", false, false, false, false)
      local mf2list = GetSelectedUnits() or {}
      for i, unit in mf2list do
        if unit:IsDead() then
          mf2list[i] = nil
        else
          fab2List[unit:GetEntityId()] = unit
          if AutoUpdateFabs then
            fabList[unit:GetEntityId()] = unit
          end
        end
      end
      SelectUnits(selection)
		end
		cleanLists(fab2List)
		totalOFabs:SetText(table.getsize(fab2List) or 0)
		
		cleanLists(fabList)
		totalFabs:SetText(table.getsize(fabList) or 0)
				
		if table.getsize(fabList) > 0 then
			local econData = GetEconomyTotals()
			local MassStorageRatio = econData['stored']['MASS']/econData['maxStorage']['MASS']
			local EnergyStorageRatio = econData['stored']['ENERGY']/econData['maxStorage']['ENERGY']
			local EnergyTrend = (econData['income']['ENERGY'] - econData['lastUseActual']['ENERGY'])*10
			local EnergyStored = econData['stored']['ENERGY']
			if MassStorageRatio == 1 then
				if EnergyTrend < 0 then
					for i, fab in fabList do
						if not GetScriptBit({fab}, 4) then
							local thisFabEnergyUse = 3500
							if fab:IsInCategory('TECH2') then
								thisFabEnergyUse = 150
							end
							if EnergyStored + ((EnergyTrend - thisFabEnergyUse) * OffTime) < 0 then
								ToggleScriptBit({fab}, 4, false)
								WaitSeconds(0.6)
								local econDataNew = GetEconomyTotals()
								local EnergyTrendNew = (econDataNew['income']['ENERGY'] - econDataNew['lastUseActual']['ENERGY'])*10
								local EnergyStoredNew = econDataNew['stored']['ENERGY']
								if EnergyTrendNew > 0 then
									break
								end
							end
						end
					end
				end
			else
				-- If energy stored will last longer than OnTime seconds and energy storage ratio is greater than  then OnStorage check if we can turn any fabs on
				if EnergyStored + (EnergyTrend * OnTime) > 0 and EnergyStorageRatio > OnStorage then
					for i, fab in fabList do
						if GetScriptBit({fab}, 4) then
							local thisFabEnergyUse = 3500
							if fab:IsInCategory('TECH2') then
								thisFabEnergyUse = 150
							end
							if EnergyStored + ((EnergyTrend - thisFabEnergyUse) * OnTime) > 0 then
								ToggleScriptBit({fab}, 4, true)
								WaitSeconds(0.6)
								local econDataNew = GetEconomyTotals()
								local EnergyTrendNew = (econDataNew['income']['ENERGY'] - econDataNew['lastUseActual']['ENERGY'])*10
								if EnergyTrendNew < 0 then
									break
								end
							end
						end
					end
				-- If energy stored will last less than 4 seconds or energy storage ratio is less than 0.1 then panic, turn all fabs off
				elseif EnergyStored + (EnergyTrend * 4) < 0 or EnergyStorageRatio < 0.1 then
					for i, fab in fabList do
						ToggleScriptBit({fab}, 4, false)
					end
				-- If energy stored will last less than OffTime seconds or energy storage ratio is less than OffStorage then start turning fabs off
				elseif EnergyStored + (EnergyTrend * OffTime) < 0 or EnergyStorageRatio < OffStorage then
					for i, fab in fabList do
						if not GetScriptBit({fab}, 4) then
							local thisFabEnergyUse = 3500
							if fab:IsInCategory('TECH2') then
								thisFabEnergyUse = 150
							end
							if EnergyStored + ((EnergyTrend - thisFabEnergyUse) * OffTime) < 0 then
								ToggleScriptBit({fab}, 4, false)
								WaitSeconds(0.6)
								local econDataNew = GetEconomyTotals()
								local EnergyTrendNew = (econDataNew['income']['ENERGY'] - econDataNew['lastUseActual']['ENERGY'])*10
								local EnergyStorageRationNew = econData['stored']['ENERGY']/econData['maxStorage']['ENERGY']
								if EnergyTrendNew > 0 then
									break
								elseif EnergyStorageRationNew < 0.05 then
									for i2, fab2 in fabList do
										ToggleScriptBit({fab2}, 4, false)
									end
									break
								end
							end
						end
					end
				end
			end
			local totalOn = 0
			for i, fab in fabList do
        if fab:IsDead() then
          fabList[i] = nil
        else
          if not GetScriptBit({fab}, 4) then
            totalOn = totalOn + 1
          end
        end
			end
			percentOnSlider:SetValue(totalOn/table.getsize(fabList))
		end
		WaitSeconds(2)
	end
end

function Init()
	if not savedPrefs then
		savedPrefs = {
			Left = 18,
			Top = 400,
			OnTime = 15,
			OffTime = 10,
			OnStorage = 0.8,
			OffStorage = 0.4,
			AddAllFabsButton = false,
		}
		Prefs.SetToCurrentProfile("UIMassFabManager_settings", savedPrefs)
		Prefs.SavePreferences()
	else
		OnTime = savedPrefs.OnTime
		OffTime = savedPrefs.OffTime
		OnStorage = savedPrefs.OnStorage
		OffStorage = savedPrefs.OffStorage
	end

	faxThread = ForkThread(GetFactionIcon)
end

function OptionsPanel(parent)
	local window = Bitmap(parent)
	window:SetTexture('/mods/AlsComboPack/textures/configwindow.dds')
	LayoutHelpers.AtLeftTopIn(window, parent)
	window.Depth:Set(function() return parent.Depth() + 10 end)
	
	CreateOptionsSlider(window, "OnTime", 5, 6, 1)
	CreateOptionsSlider(window, "OffTime", 5, 29, 1)
	CreateOptionsSlider(window, "OnStorage", 5, 52, 100)
	CreateOptionsSlider(window, "OffStorage", 5, 75, 100)
	
	local okButton = UIUtil.CreateButtonStd(window, '/widgets/small', 'OK', 16)
	LayoutHelpers.AtLeftTopIn(okButton, window, 160, 103)
	okButton.OnClick = function(self)
		Prefs.SetToCurrentProfile("UIMassFabManager_settings", savedPrefs)
		Prefs.SavePreferences()
		window:Destroy()
	end
	
	local cancelButton = UIUtil.CreateButtonStd(window, '/widgets/small', 'Cancel', 16)
	LayoutHelpers.AtLeftTopIn(cancelButton, window, 8, 103)
	cancelButton.OnClick = function(self)
		window:Destroy()
	end
end

function CreateOptionsSlider(parent, option, left, top, dividefactor)
	local title = UIUtil.CreateText(parent, option, 14, UIUtil.titleFont)
	LayoutHelpers.AtLeftTopIn(title, parent, left, top)
	title.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(parent, {Title = HelpText[option].Title, Body = HelpText[option].Body})
		end
	end
	local slider = IntegerSlider(parent, false, 0,100, 1, UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_over.dds'), UIUtil.SkinnableFile('/slider02/slider_btn_down.dds'), UIUtil.SkinnableFile('/slider02/slider-back_bmp.dds'))  
	LayoutHelpers.AtLeftTopIn(slider, parent, left + 85, top)
	local value = UIUtil.CreateText(parent, "0", 14, UIUtil.bodyFont)
	LayoutHelpers.RightOf(value, slider)
	slider:SetValue(savedPrefs[option]*dividefactor)
	value:SetText(math.floor(savedPrefs[option]*dividefactor+0.5))
	slider.OnValueChanged = function(self, newValue)
		savedPrefs[option] = newValue/dividefactor
		value:SetText(newValue)
	end
end